<?php

namespace Drupal\aiprompt\Plugin;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for AI Prompt plugins.
 */
abstract class AIPromptSegmentBase extends PluginBase implements AIPromptSegmentInterface {

  use StringTranslationTrait;
  use DependencySerializationTrait;

  /**
  * The entity type manager.
  *
  * @var \Drupal\Core\Entity\EntityTypeManagerInterface
  */
  protected $entityTypeManager;

  /**
   * Constructs a new ExamplePluginBase.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static (
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function baseDefaults(): array {
    return [
      'weight' => [
        'default' => 0,
      ],
      'status' => [
        'default' => TRUE
      ],
      'notes' => [
        'default' => ''
      ]
    ];
  }

  /**
   * {@inheritdoc}
   */
  abstract public function defineConfiguration(): array;

  /**
   * {@inheritdoc}
   */
  public function getDefaultConfiguration(): array {
    $base_defaults = $this->baseDefaults();
    $plugin_defaults = $this->defineConfiguration();

    $defaults = $this->getArrayColumn($base_defaults, 'default');
    $defaults['settings'] = $this->getArrayColumn($plugin_defaults, 'default');
    return $defaults;
  }

  /**
   * {@inheritdoc}
   */
  public function getPluginConfig(): mixed {
    return $this->getConfigurationValue('settings');
  }

  /**
   * {@inheritdoc}
   */
  public function getPluginConfigValue(int|string $key): mixed {
    return $this->getConfigurationValue(['settings', $key]);
  }

  /**
   * {@inheritdoc}
   */
  public function &getConfigurationValue(mixed $key): mixed {

    // get saved configuration
    $value =& NestedArray::getValue($this->configuration, (array) $key);

    // get default configuration
    if ($value === NULL) {
      $defaults = $this->getDefaultConfiguration();
      $value =& NestedArray::getValue($defaults, (array) $key);
    }

    return $value;
  }

  /**
   * {@inheritdoc}
   */
  public function getBaseConfigKeys(): array {
    $default = $this->baseDefaults();
    return array_keys($default);
  }

  /**
   * {@inheritdoc}
   */
  public function getPluginConfigKeys(): array {
    $default = $this->defineConfiguration();
    return array_keys($default);
  }

  /**
   * {@inheritdoc}
   */
  abstract public function render(): string;

  /**
   * Generate output string for preview
   *
   * @return string
   */
  protected function preview(): string {
    $string = $this->render();

    $max_length = 3000;

    if (strlen($string) > $max_length) {
      $string = mb_substr($string, 0, $max_length) . '...';
    }

    return $string;
  }

  /**
   * {@inheritdoc}
   */
  public function buildPreview(): array {
    $string = $this->preview();
    return [
      '#type' => 'markup',
      '#markup' => nl2br(Html::escape($string)),
      '#allowed_tags' => ['br']
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {

    $form['weight'] = [
      '#type' => 'weight',
      '#title' => $this->t('Weight'),
      '#default_value' => $this->getConfigurationValue('weight'),
      '#description' => $this->t('When listing segments, those with smaller weights will be displayed first.'),
      '#weight' => 5
    ];
    
    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $this->getConfigurationValue('status'),
      '#description' => $this->t('Uncheck this box to disable this prompt segment.'),
      '#weight' => 4
    ];

    $form['notes'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Administrative notes'),
      '#default_value' => $this->getConfigurationValue('notes'),
      '#required' => FALSE,
      '#weight' => -5
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state): void {

  }

  /**
   * {@inheritdoc}
   */
  public function getStatus(): bool {
    return (bool) $this->getConfigurationValue('status');
  }

  /**
   * {@inheritdoc}
   */
  public function getWeight(): int {
    return (int) $this->getConfigurationValue('weight');
  }

  /**
   * {@inheritdoc}
   */
  public function getAIPromptId(): int|string {
    return $this->getPluginConfigValue('aiprompt_id');
  }

  /**
   * Helper function to extract a column from array.
   */
  private function getArrayColumn(&$array, $key): array {
    return array_combine(array_keys($array), array_column($array, $key));
  }
}
